﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AirPlaneWithRadar
{
    public class DrawingPlain
    {
        public EntetyPlain Plain { get; protected set; }
        protected float startPosX;
        protected float startPosY;
        private int? pictureWidth = null;
        private int? pictureHeight = null;
        protected readonly int plainWidth = 120;
        protected readonly int plainHeight = 60;
        public DrawingPlain(int speed, float weight, Color bodycolor)
        {
            Plain = new EntetyPlain(speed, weight, bodycolor);
        }
        protected DrawingPlain(int speed, float weight, Color bodyColor, int plainWidth, int plainHeight) :
             this(speed, weight, bodyColor)
        {
            this.plainWidth = plainWidth;
            this.plainHeight = plainHeight;
        }
        public void ReColor(Color col)
        {
            Plain.BodyColor = col;
        }

        public void setPosition(int x, int y, int width, int height)
        {
            if (x + plainWidth > width || y + plainHeight > height || plainHeight > height || plainWidth > width || x < 0 || y < 0)
                return;
            else
            {
                startPosX = x;
                startPosY = y;
                pictureWidth = width;
                pictureHeight = height;
            }
        }
        public void MoveTransport(Direction direction)
        {
            if (!pictureWidth.HasValue || !pictureHeight.HasValue)
            { return; }
            switch (direction)
            {
                case Direction.Right:
                    if (startPosX + plainWidth + Plain.Step < pictureWidth)
                        startPosX += Plain.Step;
                    break;

                case Direction.Left:
                    if (startPosX - Plain.Step > 0)
                        startPosX -= Plain.Step;
                    break;
                case Direction.Down:
                    if (startPosY + plainHeight + Plain.Step < pictureHeight)
                        startPosY += Plain.Step;
                    break;
                case Direction.Up:
                    if (startPosY - Plain.Step > 0)
                        startPosY -= Plain.Step;
                    break;
            }
        }
        public virtual void DrawTransoprt(Graphics g)
        {
            if (startPosX < 0 || startPosY < 0 || !pictureHeight.HasValue || !pictureWidth.HasValue)
            {
                return;
            }

            Pen pen = new Pen(Color.Black);


            g.DrawRectangle(pen, startPosX, startPosY, 20, 30);
            g.DrawRectangle(pen, startPosX, startPosY + 30, 100, 30);
            g.DrawRectangle(pen, startPosX + 100, startPosY + 40, 20, 15);
            //koleso1
            g.DrawRectangle(pen, startPosX + 30, startPosY + 60, 5, 10);
            g.DrawEllipse(pen, startPosX + 28, startPosY + 70, 9, 9);
            //koleso2
            g.DrawRectangle(pen, startPosX + 80, startPosY + 60, 5, 10);
            g.DrawEllipse(pen, startPosX + 78, startPosY + 70, 9, 9);


            //Korpys
            Brush br = new SolidBrush(Plain?.BodyColor ?? Color.Black);
            g.FillRectangle(br, startPosX + 3, startPosY + 33, 94, 24);
            g.FillRectangle(br, startPosX + 1, startPosY + 1, 19, 29);


            //krilya
            Brush brWings = new SolidBrush(Color.Black);
            g.FillRectangle(brWings, startPosX + 30, startPosY + 40, 40, 8);


            //cabina
            Brush brCabine = new SolidBrush(Color.Blue);
            g.FillRectangle(brCabine, startPosX + 101, startPosY + 41, 19, 14);





        }
        public void ChangeBorders(int width, int height)
        {
            pictureWidth = width;
            pictureHeight = height;
            if (pictureWidth < plainWidth || pictureHeight < plainHeight)
            {
                pictureWidth = null;
                pictureHeight = null;
                return;
            }
            if (startPosX + plainWidth > pictureWidth)
                startPosX = pictureWidth.Value - plainWidth;

            if (startPosY + plainHeight > pictureHeight)
                startPosY = pictureHeight.Value - plainHeight;
        }
        public (float Left, float Right, float Top, float Bottom) GetCurrentPosition()
        {
            return (startPosX, startPosY, startPosX + plainWidth, startPosY + plainHeight);
        }
    }
}
