﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace AirPlaneWithRadar
{
    public partial class FormPlaneConfig : Form
    {
        DrawingPlain _Plane = null;

        private event Action<DrawingPlain> EventAddPlane;

        public FormPlaneConfig()
        {
            InitializeComponent();
            panelBlack.MouseDown += PanelColor_MouseDown;
            panelPurple.MouseDown += PanelColor_MouseDown;
            panelGray.MouseDown += PanelColor_MouseDown;
            panelGreen.MouseDown += PanelColor_MouseDown;
            panelRed.MouseDown += PanelColor_MouseDown;
            panelWhite.MouseDown += PanelColor_MouseDown;
            panelYellow.MouseDown += PanelColor_MouseDown;
            panelBlue.MouseDown += PanelColor_MouseDown;
            buttonCancel.Click += (object sender, EventArgs e) => { Close(); };

        }

        private void DrawPlane()
        {
            Bitmap bmp = new(pictureBoxObject.Width, pictureBoxObject.Height);
            Graphics gr = Graphics.FromImage(bmp);
            _Plane?.setPosition(5, 5, pictureBoxObject.Width, pictureBoxObject.Height);
            _Plane?.DrawTransoprt(gr);
            pictureBoxObject.Image = bmp;
        }

        public void AddEvent(Action<DrawingPlain> ev)
        {
            if (EventAddPlane == null)
            {
                EventAddPlane = ev;
            }
            else
            {
                EventAddPlane += ev;
            }
        }

        private void LabelObject_MouseDown(object sender, MouseEventArgs e)
        {
            (sender as Label).DoDragDrop((sender as Label).Name, DragDropEffects.Move | DragDropEffects.Copy);
        }

        private void PanelObject_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.Text))
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }

        private void PanelObject_DragDrop(object sender, DragEventArgs e)
        {
            if (_Plane != null)
            {
                labelBaseColor.BackColor = Color.Transparent;
                labelDopColor.BackColor = Color.Transparent;
            }

            switch (e.Data.GetData(DataFormats.Text).ToString())
            {
                case "labelSimpleObject":
                    _Plane = new DrawingPlain((int)numericUpDownSpeed.Value, (int)numericUpDownWeight.Value, Color.White);
                    break;
                case "labelModifiedObject":
                    _Plane = new DrawingRadarPlane((int)numericUpDownSpeed.Value, (int)numericUpDownWeight.Value, Color.White, Color.Black,
                        checkBoxRadar.Checked, checkBoxOilBox.Checked);
                    break;
            }
            DrawPlane();
        }

        private void PanelColor_MouseDown(object sender, MouseEventArgs e)
        {
            (sender as Control).DoDragDrop((sender as Control).BackColor, DragDropEffects.Move | DragDropEffects.Copy);
        }

        private void LabelBaseColor_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(typeof(Color)))
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }
        private void LabelDopColor_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(typeof(Color)))
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }

        private void LabelBaseColor_DragDrop(object sender, DragEventArgs e)
        {
            if (_Plane is null) return;
            var col = e.Data.GetData(typeof(Color));
            Color newCol = (Color)col;
            labelBaseColor.BackColor = newCol;
            _Plane?.ReColor(newCol);
            DrawPlane();
        }

        private void LabelDopColor_DragDrop(object sender, DragEventArgs e)
        {
            if (_Plane is DrawingRadarPlane)
            {
                var col = e.Data.GetData(typeof(Color));
                Color newCol = (Color)col;
                labelDopColor.BackColor = newCol;
                (_Plane as DrawingRadarPlane).ReColorDop(newCol);
                DrawPlane();
            }

        }

        private void ButtonOk_Click(object sender, EventArgs e)
        {
            EventAddPlane?.Invoke(_Plane);
            Close();
        }

    }
}
